/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.exchange.httptransport;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.net.Socket;

/**
 * This class provide methods
 * to access exchange server datastore
 * by socket connection
 *
 * @author Fabio Maggi @ Funambol
 * @version $Id: WebDavHttpTransport.java,v 1.6 2005/01/19 11:37:47 fabius Exp $
 **/
public class WebDavHttpTransport {

    //---------------------------------------------------------------- Constants

    private static final String NEW_LINE = "\r\n" ;

    //---------------------------------------------------------------- Private data

    private String  host  = null ;
    private int     port  = 0    ;

    //---------------------------------------------------------------- Costructors

    public WebDavHttpTransport(String host,
                               int    port) {

        this.host = host ;
        this.port = port ;
    }

    //---------------------------------------------------------------- Public methods

    /**
     * Send request to exchange server
     * and get response
     *
     * @param request
     * @return response
     *
     * @throws IOException
     **/
    public String sendRequest (String request)

        throws IOException {

        Socket               s         = null ;
        BufferedReader       in        = null ;
        DataOutputStream     out       = null ;
        InputStream          is        = null ;

        String               line      = null ;
        StringBuffer         response  = null ;

        //System.out.println("debug - request:" + request);

        try {

            s   = new Socket(this.host,
                             this.port) ;

            is  = s.getInputStream()                           ;
            in  = new BufferedReader(new InputStreamReader (is))        ;
            out = new DataOutputStream(s.getOutputStream())    ;

            out.writeBytes(request  +
                           NEW_LINE +
                           NEW_LINE);
            out.flush();

            response = new StringBuffer();

            //System.out.println("debug - response:");
            while ((line = in.readLine()) != null){
                 //System.out.println(line);
                 response.append(line);
            }

            return response.toString();

        } finally {

            if (out != null) {
                out.close () ;
            }
            if (in != null)  {
                in.close  () ;
            }
            if (in != null)  {
                is.close  () ;
            }
            if (in != null)  {
                s.close  ()  ;
            }
        }

    }

}